const fs = require('fs');
const path = require('path');
const express = require('express');
const bodyParser = require('body-parser');
const { DotLottieUtils } = require('@lottiefiles/dotlottie-js');

const imageminPngquant = require('imagemin-pngquant');
const pngToJpeg = require('png-to-jpeg');
var PNG = require('pngjs').PNG;

const PORT = 7071;
let lastCode = '';

const app = express.createServer();
app.use(bodyParser.json({ limit: '50mb' }));

app.use(function (req, res, next) {
  res.header('Content-Type', 'application/json');
  res.header('Access-Control-Allow-Origin', '*');
  res.header('Access-Control-Allow-Headers', 'Origin, X-Requested-With, Content-Type, Accept');
  next();
});

app.get('/callback', function (req, res) {
  if (req.query.code) {
    lastCode = req.query.code;
  }

  res.send(JSON.stringify({ success: true }));
});

app.get('/code', function (req, res) {
  res.send(JSON.stringify({ code: lastCode }));
});

app.post('/convert-bodymovin', async function (req, res) {
  const { data, filePath } = req.body;

  try {
    fs.writeFile(path.resolve(filePath), JSON.stringify(data), 'utf8', (error, success) => {
      res.send({
        status: 'success',
      });
    });
  } catch (error) {
    res.send({
      status: 'error',
      details: error.toString(),
    });
  }
});

// app.post('/convert-tgs', async function(req, res) {
//   const { data, filePath } = req.body;
//
//   try {
//     // const tgs = new TGSKit();
//     await tgs.load(data);
//
//     const errors = anim.validate();
//
//     if (errors.length === 0) {
//       fs.writeFile(path.resolve(filePath), tgs.generate(), 'binary', (error, success) => {
//         res.send({
//           status: 'success',
//         });
//       });
//     } else {
//       res.send({
//         status: 'error',
//         details: errors,
//       });
//     }
//   } catch (error) {
//     res.send({
//       status: 'error',
//       details: error.toString(),
//     });
//   }
// });

app.post('/convert-dotlottie', async function (req, res) {
  const { data, filePath } = req.body;
  try {
    const animationsData = { loop: true, id: 'animation_1', speed: 1.0 };

    if (data.meta && data.meta.tc) {
      animationsData.themeColor = data.meta.tc;
    }

    const manifest = {
      animations: [animationsData],
      version: '1.0',
    };

    if (data.meta) {
      if (data.meta.a) {
        manifest.author = data.meta.a;
      }
      if (data.meta.d) {
        manifest.description = data.meta.d;
      }
      if (data.meta.g) {
        manifest.generator = data.meta.g;
      }
      if (data.meta.k) {
        manifest.keywords = data.meta.k;
      }
    }

    const dotLottie = await DotLottieUtils.build('');
    dotLottie.setManifest(manifest);
    dotLottie.addAnimation('animation_1', JSON.stringify(data));
    await dotLottie.createDotLottie(filePath);
    res.send({
      status: 'success',
    });
  } catch (error) {
    res.send({
      status: 'error',
      details: error.toString(),
    });
  }
});

// Process image helpers
async function processImage(path, compression, hasTransparency) {
  const destinationPathFolder = path.substr(0, path.lastIndexOf('/') + 1);
  const destinationFullPath = destinationPathFolder;
  const plugins = [];
  if (hasTransparency) {
    plugins.push(
      imageminPngquant({
        quality: [0, compression],
      }),
    );
  } else {
    plugins.push(pngToJpeg({ quality: Math.round(compression * 100) }));
  }
  const files = await imagemin([path], {
    destination: destinationFullPath,
    plugins,
  });

  return files;
}

function checkImageTransparency(imagePath) {
  return new Promise((resolve, reject) => {
    try {
      const stream = fs.createReadStream(imagePath);
      stream.on('error', function (error) {
        reject(error);
      });
      const pngStream = stream.pipe(new PNG());

      pngStream.on('metadata', function (meta) {});

      pngStream.on('parsed', function () {
        var hasTransparency = false;
        for (var y = 0; y < this.height; y++) {
          for (var x = 0; x < this.width; x++) {
            var idx = (this.width * y + x) << 2;
            if (this.data[idx + 3] !== 255) {
              hasTransparency = true;
              x = this.width;
              y = this.height;
            }
          }
        }
        resolve(hasTransparency);
      });
    } catch (err) {
      reject(err);
    }
  });
}

app.post('/encode', async function (req, res) {
  if (req.body.path) {
    try {
      const fs = require('fs');
      const decodedPath = decodeURIComponent(req.body.path);

      const buff = fs.readFileSync(decodedPath);
      const base64data = buff.toString('base64');
      res.send({
        status: 'success',
        data: base64data,
      });
    } catch (err) {
      res.send({
        status: 'error',
        message: 'failed decoding',
        error: err,
        errorMessage: err.message,
      });
    }
  } else {
    res.send({
      status: 'error',
      message: 'missing params',
    });
  }
});

app.post('/getType', async function (req, res) {
  if (req.body.path) {
    try {
      const fileType = await FileType.fromFile(decodeURIComponent(req.body.path));
      res.send({
        status: 'success',
        fileType: fileType,
      });
    } catch (err) {
      res.send({
        status: 'error',
        message: 'failed getting type',
        error: err,
        errorMessage: err.message,
      });
    }
  } else {
    res.send({
      status: 'error',
      message: 'missing params',
    });
  }
});

app.post('/processImage', async function (req, res) {
  if (req.body.path && req.body.compression) {
    try {
      const decodedPath = decodeURIComponent(req.body.path);
      const hasTransparency = await checkImageTransparency(decodedPath);
      const processedImages = await processImage(decodedPath, req.body.compression, hasTransparency);
      if (!hasTransparency) {
        var renamedPath = decodedPath.substr(0, decodedPath.lastIndexOf('.png')) + '.jpg';
        fs.renameSync(decodedPath, renamedPath);
      }
      if (processedImages.length) {
        res.send({
          status: 'success',
          path: processedImages[0].destinationPath,
          extension: hasTransparency ? 'png' : 'jpg',
        });
      } else {
        res.send({
          status: 'error',
          message: 'Could not export',
        });
      }
    } catch (error) {
      res.send({
        status: 'error',
        err: error,
        message: error.message,
      });
    }
  } else {
    res.send({
      status: 'error',
      message: 'missing params',
    });
  }
});

app.listen(PORT);
