# dotlottie abstraction library

This library is designed to make your interaction with dotLottie easier.

### Usage

1. Install

```
yarn add @lottiefiles/dotlottie-utils@0.0.1
```

2. Use

```
import { DotLottieUtils } from '@lottiefiles/dotlottie-toolkit';

const dotLottie = await DotLottieUtils.build(url);
```

### Documentation &amp; API

#### Full documentation is available here [Click me](https://docs.lottiefiles.com/dotlottie-js/)

#### `setAnimations(animations: string[])`

Sets the animations array. consists of json stringified Lotties

#### `addAnimation(animation: string)`

Adds a json stringified Lottie animation to the animations array

#### `removeAnimation(index: number)`

Remove an animation from animations array given the index

#### `getAnimation(index: number) => string`

Gets a single animation from animations array given the index

#### `getAnimations() => string[]`

Get all animations inside the dotLottie file

#### `setManifest(object:object)`

set Manifest object. the structure must be as follows

```javascript
{
      animations: [{ loop: true, themeColor: '#ffffff', id: 'animation', speed: 1.0 }],
      author: 'my name',
      description: 'whats in here',
      generator: 'generated by',
      keywords: 'keywords',
      version: 'version number',
    }
```

#### `fromURL(url:string) => Promise<any>`

Returns an object consisting of an array of json string Lottie animations and the manifest object

#### `toString(url:string) => Promise<string>`

Converts the dotLottie file into a dataURI and returns it

#### `toString(url:string) => Promise<ArrayBuffer>`

Converts the dotLottie file into an array buffer and returns it

#### `createDotLottie(filePath:string) => Promise<ArrayBuffer>`

Creates the dotLottie file given the file path. its important to set the animations and manifest file and any additional
variables before calling this method

### Development

---

Development work on the toolkit requires Nodejs and Yarn.

#### Guidelines

- Use [defensive programming](https://en.wikipedia.org/wiki/Defensive_programming) techniques: Ensure type and range of
  input values, cast values to native representation whenever possible, etc.
- Refrain from using external dependencies: Discuss before adding a dependency. Check with
  [Bundlephobia](https://bundlephobia.com/) for package size and dependencies when choosing one.
- Use code formatting in the IDE using the given eslint+prettier configs.
- Write tests to cover all functions and code branches with valid and invalid values.

#### Setting up

```
git clone https://github.com/LottieFiles/dotottie-utils

cd lottie-js

yarn install
```

#### Running test suite

```
yarn test
```

#### Building

```
yarn build
```
